function ProportionalLegend(canvasWidth, canvasHeight) {"use strict";

    // font size for text labels.
    var fontSize = 13,

    // minimum distance between two neighboring values. Expressed as a percentage relative to the font size.
    minDistancePerc = 100,

    // minimum value represented in the legend.
    minimum = 1234,

    // maximum value represented in the legend.
    maximum = 123456,

    // radius of the circle for the maximum value
    maximumRadius = 100,

    // scale factor to down-scale the legend. The radius of the largest circle drawn is maximumRadius x sizeScalePerc / 100
    sizeScalePerc = 100,

    // place lines and labels at values starting with these numbers. 5, 2.5, 1 are the recommended first digits of the default breaks.
    bases = [5, 2.5, 1];

    this.setSizeScalePerc = function(sScalePerc) {
        sizeScalePerc = sScalePerc;
    };

    this.getSizeScalePerc = function() {
        return sizeScalePerc;
    };

    this.getMinimum = function() {
        return minimum;
    };

    this.setMinimum = function(min) {
        minimum = min;
    };

    this.getMaximum = function() {
        return maximum;
    };

    this.setMaximum = function(max) {
        maximum = max;
    };

    this.setBases = function(bas) {
        bases = bas;
    };
    
     this.getBasesString = function() {
        var i, str = "";
        for (i = 0; i < bases.length - 1; i += 1) {
            str += bases[i] + " ";
        }
        if (bases.length > 0) {
            str += bases[bases.length - 1];
        }
        return str;
    };

    this.setMinDistancePerc = function(minLimDistPerc) {
        minDistancePerc = minLimDistPerc;
    };

    this.getMinDistancePerc = function() {
        return minDistancePerc;
    };

    this.setFontSize = function(fontS) {
        fontSize = fontS;
    };

    this.getFontSize = function() {
        return fontSize;
    };
    
    this.setMaximumRadius = function(r) {
        maximumRadius = r;
    };

    /**
     * Converts a value to a symbol height. For a circle this is twice its radius.
     */
    function valueToSymbolHeight(val) {
        return Math.sqrt(val / maximum) * maximumRadius * sizeScalePerc / 100 * 2;
    }


    this.valueToSymbolHeight = valueToSymbolHeight;

    function log10(val) {
        return Math.log(val) / Math.LN10;
    }

    /**
     * Computes the values for drawing the legend. The new values are returned
     * in an array of doubles. Values are ordered in decreasing order.
     * @return An array with the potential values.
     */
    function computePotentialValues() {
        var min, max, baseID, values, scaleExp, scale, min_s, max_s, ndigits, firstLimit, i, v;

        // make sure the minimum is larger than 0
        min = Math.max(1e-10, Math.abs(minimum));
        max = Math.abs(maximum);
        if (isNaN(min) || isNaN(max)) {
            return [];
        }
        if (min === max) {
            return [max];
        }

        // an index into bases array
        baseID = 0;

        // array that will hold the potential values
        values = [];

        // add the maximum value to the array
        values[0] = max;

        /* compute the scale factor to guarantee that minimum > 1
         This is possibly more stable than:
         while (min < 1.) {
         .    min *= 10.;
         .    max *= 10.;
         .    scale /= 10;
         }*/
        scaleExp = Math.ceil(-log10(min));
        scale = Math.pow(10, scaleExp);
        min_s = min * scale;
        max_s = max * scale;

        // compute the number of digits of the scaled maximum value
        ndigits = Math.floor(log10(max_s));

        // find the index into the bases array for the first v smaller than max_s
        firstLimit = max_s / Math.pow(10, ndigits);
        for ( i = 0; i < bases.length; i += 1) {

            if (firstLimit >= bases[i]) {
                baseID = i;
                break;
            }
        }

        // compute the values following the maximum
        while (true) {
            // compute the value
            v = min_s;
            if (baseID < bases.length) {
                v = bases[baseID] * Math.pow(10, ndigits);
            }

            // if the value is smaller than min_s, add the minimum and stop the loop
            if (v <= min_s) {
                // add minimum value
                values.push(min);
                break;
            }

            // otherwise divide v by the scale. This could be done with:
            // v /= scale;
            // However, this is numerically less precise and sometimes leads
            // to rounding errors due to the limited precision of floating point
            // values. The following alternative is numerically more stable:
            v = bases[baseID] * Math.pow(10, (ndigits - scaleExp));

            //  store the scaled value in the values array
            values.push(v);

            // switch to the next base
            baseID += 1;
            if (baseID >= bases.length) {
                baseID = 0;
                ndigits -= 1;
            }
        }
        return values;
    }

    /**
     * Filters the passed candidate values, making sure values are not too close
     * to each other when rendered.
     * @param values All unfiltered values.
     * @param dmin The minimum size difference between two symbols.
     * @return An array with the filtered values.
     */
    function filterValues(values, dmin) {
        var filteredValues, lastHeight, limitID, lastValueID, previousHeight, v, h, closeToLast, closeToPrevious;

        // array that will hold the filtered values
        filteredValues = [];

        // add the maximum value
        filteredValues.push(values[0]);

        // find the height and value of the smallest acceptable symbol
        lastHeight = 0;
        lastValueID = values.length - 1;
        // start with the minimum value
        while (lastValueID >= 0) {
            lastHeight = valueToSymbolHeight(values[lastValueID]);
            if (lastHeight > dmin) {
                break;
            }
            lastValueID -= 1;
        }

        // loop over all values that are large enough
        for ( limitID = 1; limitID <= lastValueID; limitID += 1) {
            v = values[limitID];

            // compute the height of the symbol
            h = valueToSymbolHeight(v);
            previousHeight = valueToSymbolHeight(filteredValues[filteredValues.length - 1]);

            // do not add the symbol if it is too close to the smallest symbol
            // (but is not the smallest symbol itself)
            closeToLast = (h - lastHeight) < dmin && limitID !== lastValueID;

            // do not add the symbol if it is too close to the previously
            // drawn symbol
            closeToPrevious = (previousHeight - h) < dmin;

            if (!closeToLast && !closeToPrevious) {
                filteredValues.push(values[limitID]);
            }
        }

        return filteredValues;

    }


    this.getValues = function() {
        var dmin, potentialValues;

        // compute all potential values in the mapped range.
        potentialValues = computePotentialValues();

        // the minimum vertical distance between two circles in pixel
        dmin = fontSize * minDistancePerc / 100;
        
        // filter the values: values must not be too close to each other.
        return filterValues(potentialValues, dmin);
    };
}